#ifndef _azpforg_h
#define _azpforg_h

/**/
/* Azpforg 3D engine by Motzel - main header file.                          */
/**/
/* Working log :                                                            */
/*   04.07.97 - 12:00  16:00                                              */
/*               21:00  02:00                                              */
/*   07.07.97 - 17:30  18:30                                              */
/*   09.07.97 - 10:00  11:30                                              */
/*               16:00  20:00                                              */
/*   11.07.97 - 22:00  00:30                                              */
/*   12.07.97 - 11:00  12:30                                              */
/*               14:00  00:10                                              */
/*   13.07.97   09:00  11:00                                              */
/*               11:30  17:30                                              */
/*   14,15.07   No coding - HDD crash :(                                   */
/*               Fortunately I've made backup of engine a day before... :)  */
/*   16.07.97   12:30  19:00                                              */
/*   17.07.97   15:00  21:00                                              */
/*   18.07.97   07:30  12:30                                              */
/*               15:00  17:30                                              */
/*   20.07.97   11:00  14:00                                              */
/*               15:30  18:00                                              */
/*   14.08.97   12:00  16:00                                              */
/*   15.08.97   11:00  12:00                                              */
/*   20.08.97   12:00  16:00 ( 3DS To MZL converter )                     */
/*   12.09.97   12:00  18:00                                              */
/*   14.09.97   15:00  18:00                                              */
/*   15.09.97   14:00  18:00 ( Keyboard handler )                         */
/*   22.09.97   14:00  17:00                                              */
/*   01.10.97   11:00  14:00                                              */
/*                                                                          */
/* Note :                                                                   */
/* It's first program I've ever written in "C" ! (except "hello.c" of coz :)*/
/* It's my first 3D engine, too.                                            */
/**/


/**/
/*                          Wlaczenie includow                              */
/**/
#include "my_gfx.h"     /* VGA lib                                          */

/**/
/*                         Definicje struktur danych                        */
/**/
/* ======================= Definicje struktur 3D ========================== */
typedef double matrix[4][4];   /* Macierz 4x4 */

typedef struct                 /* Punkt 3D i wektor 3D */
{
        double x, y, z;
}point_3d, vect3d;

typedef struct                 /* Punkt 2D */
{
        int x, y;
}point_2d;

typedef struct {               /* Trojkat (sciana obiektu)                  */
  int p1, p2, p3;              /* Indeksy do wierzcholkow trojkata          */
  int c;                       /* Kolor sciany                              */
  int c1, c2, c3;              /* Kolory w wierzcholkach (cieniowanie)      */
  BYTE u1, v1, u2, v2, u3, v3; /* Koordynaty tekstury                       */
  vect3d fnormal;              /* Normalna do trojkata                      */
  vect3d fnormal_rot;          /* Normalna do trojkata po przeksztalceniach */
  double average_z;            /* Srednia z wartosci wsp. Z do sortowania   */
  int    obj_num;              /* Numer obiektu, do ktorego ta sciana nalezy*/
  BYTE   face_type;            /* Typ sciany FLAT_FACE lub TEXTURE_FACE     */
  BYTE   face_shading;         /* Typ cieniowania                           */
  BYTE   flags;                /* Flagi sciany                              */
  BYTE   *texture;             /* Wskaznik na teksture lub phongmap obiektu */
}triangle;

typedef struct {               /* Obiekt 3D */
  double   radius;             /* Promien obiektu */
  point_3d position;           /* Wspolrzedne obiektu w przestrzeni swiata  */
  BYTE     angle_x, angle_y,
           angle_z;            /* Katy obrotu wzgledem osi X, Y i Z */
  int      num_vert;           /* Ilosc punktow  */
  point_3d *vert;              /* Wspolrzedne punktow w przestrzeni obiektu */
  point_3d *vert_rot;          /* Wspolrzedne po przeksztalceniach          */
  point_2d *vert2d;            /* Tablica ze wspolrzednymi ekranowymi       */
  vect3d   *vert_norm;         /* Tablica normalnych do punktow             */
  vect3d   *vert_norm_rot;     /* Tablica normalnych po przeksztalceniach   */
  matrix   matrix_o;           /* Macierz obiektu                           */
  matrix   matrix_on;          /* Macierz obiektu dla normalnych            */
  int      num_face;           /* Ilosc trojkatow w obiekcie                */
  triangle *faces;             /* Trojkaty, z ktorych sklada sie obiekt     */
  BYTE     flags;              /* Flagi obiektu                             */
  BYTE     first_col,
           num_shades;         /* Pierwszy kolor i liczba odcieni           */
}object3d;

typedef struct {               /* Kamera                                    */
  point_3d position;           /* Wspolrzedne kamery                        */
  point_3d inv_position;
  vect3d   view_vect;          /* Wektor kamery                             */
  BYTE     angle_x, angle_y,
           angle_z;            /* Katy obrotu kamery                        */
  matrix   matrix_c;           /* Macierz kamery                            */
  matrix   matrix_cn;          /* Macierz kamery do normalnych              */
}camera;

/* ================= Definicje struktur plikowych do 3D =================== */
typedef struct                 /* Naglowek pliku MZL                        */
{
  char c1, c2, c3;             /* Litery MZL                                */
  struct {
    BYTE vert_exist      : 1;  /* W pliku istnieje informacja o punktach    */
    BYTE face_exist      : 1;  /* Jest info o scianach                      */
    BYTE face_norm_exist : 1;  /* Jest info o normalnych scian              */
    BYTE vert_norm_exist : 1;  /* Jest info o normalnych punktow            */
    BYTE text_exist      : 1;  /* Jest info o koordynatach tekstury         */
    BYTE unused          : 3;  /* Nie uzywane                               */
  }flags;
}mzl_header;

typedef struct {               /* Punkt w pliku MZL                         */
  float  x, y, z;
}mzl_point3d;

typedef struct {               /* Sciana w pliku MZL                        */
  WORD  p1, p2, p3;
}mzl_face;

typedef struct {               /* Koordynaty tekstur w pliku MZL            */
  BYTE x1, y1, x2, y2, x3, y3;
}mzl_textUV;

/**/
/*                          Definicje stalych                               */
/**/
#define FALSE             0
#define TRUE              1

#define SET_MODE          1
#define AND_MODE          2
#define OR_MODE           3

#define FLAT_FACE         1
#define TEXTURE_FACE      2
#define ENVMAP_FACE       3

#define AMBIENT_SHADING   1
#define LAMBERT_SHADING   2
#define GOURAUD_SHADING   4
#define PHONG_SHADING     8

#define OBJ_VISIBLE       1
#define OBJ_CLIPPED       2
#define OBJ_TWOSIDED      4
#define OBJ_ON            8

#define FACE_VISIBLE      1
#define FACE_CLIPPED      2

#define NEAR_CLIP         3
#define FAR_CLIP          1000

/**/
/*                   Zmienne zdefiniowane w bibliotece                      */
/**/
extern double azp_SinTable[256];
extern double azp_CosTable[256];
extern int       num_world_vertices;

extern int       num_world_objects;
extern int       max_object;
extern object3d  *world;

extern int       num_world_faces;
extern int       num_visible_faces;
extern int       max_face;
extern triangle  *facelist;

extern int       num_lights;
extern vect3d    *lightlist;

extern int FOCUS;

/**/
/*                           Funkcje biblioteki                             */
/**/
/* ===================== Funkcje z modulu my_3dobj.c ====================== */
BYTE Initialize_object(object3d *obj, int num_vert, int num_face, int num_obj);
void Free_object(object3d *obj);
void Set_object_location(int num_obj, point_3d loc);
void Set_object_rotation(int num_obj, BYTE anglex, BYTE angley, BYTE anglez);
void Set_object_flags(int num_obj, BYTE flags, BYTE mode);
void Set_object_color(int num_obj, int color);
void Set_object_type(int num_obj, BYTE type);
void Set_object_shading(int num_obj, BYTE shad, BYTE first, BYTE num_shad);
void Set_object_texture(int num_obj, BYTE *tex);

void Initialize_camera(camera *cam);
void Set_camera_rotation(camera *cam, BYTE anglex, BYTE angley, BYTE anglez);
void Set_camera_location(camera *cam, point_3d loc);

void Create_object_matrix(int num_obj);
void Create_camera_matrix(camera *cam);

void Scale_object(int num_obj, double dscale);
void Scale_object_to(int num_obj, double rad);
void Calculate_object_center(int num_obj);
void Calculate_object_radius(int num_obj);
void Calculate_object_normals(int num_obj);

/* ===================== Funkcje z modulu my_mzl3d.c ====================== */
BYTE Load_mzl(char *fname, int num_obj, DWORD offset);

/* ===================== Funkcje z modulu mymatrix.c ====================== */
void Create_SinCos_Table(void);
void matrixIdentity(matrix maca);
void matrixMultiply(matrix maca, matrix macb);
void matrixTranslation(matrix maca, double dx, double dy, double dz);
void matrixScale(matrix maca, double ds);
void matrixRotateX(matrix maca, BYTE anglex);
void matrixRotateY(matrix maca, BYTE angley);
void matrixRotateZ(matrix maca, BYTE anglez);
void matrixRotateXYZ(matrix maca, BYTE anglex, BYTE angley, BYTE anglez);
void Transform_point(matrix maca, point_3d *pa, point_3d *pb, int count);
void Inverse_transform_point(matrix maca, point_3d *pa, point_3d *pb);
void Perspective(point_3d *p3d, point_2d *p2d, int count);
void matrixPrint(matrix maca);

/* ===================== Funkcje z modulu myrender.c ====================== */
BYTE   rad2ang(double rad);
double vector_dot(vect3d u, vect3d v);
void   vector_cross(vect3d u, vect3d v, vect3d *result);
double vector_length(vect3d u);
void   vector_normalize(vect3d *u);
vect3d vector_create(point_3d a, point_3d b);

void Set_light(int num_l, double x, double y, double z, double px, double py, double pz);

BYTE Init_world(int num_obj, int num_face, int num_l);
void Free_world(void);

void Render_world(camera *cam);
int  qsort_cmp(const void *e1, const void *e2);
void Sort_faces(void);
void Draw_faces(void);
void Remove_hidden_objects(camera *cam);
#endif
