#ifndef _my_gfx_h
#define _my_gfx_h

/**/
/* Ponizsze trzy makra zostaly napisane przez Shawna Hargreavesa            */
/**/
#define END_OF_FUNCTION(x)    void x##_end() { }
#define LOCK_VARIABLE(x)      _go32_dpmi_lock_data((void *)&x, sizeof(x))
#define LOCK_FUNCTION(x)      _go32_dpmi_lock_code(x, (long)x##_end - (long)x)

#define PACKED __attribute__ ((packed))

/**/
/*                         Definicje nowych typow                           */
/**/
typedef unsigned char      BYTE;
typedef unsigned short int WORD;
typedef unsigned long int  DWORD;

/**/
/*                         Definicje struktur danych                        */
/**/
/* =================== Definicje struktur graficznych ===================== */
typedef struct {               /* Definicja koloru w pliku BMP              */
  BYTE b, g, r, reserved;
}bmp_RGB;

typedef struct {               /* Wewnetrzna definicja koloru w bibliotece  */
  BYTE r, g, b;
}RGB;

typedef struct {               /* Struktura bitmapy                         */
  int  width, height;          /* Szerokosc i wysokosc bitmapy              */
  BYTE *data;                  /* Dane bitmapy                              */
}bitmap;

typedef struct {               /* Struktura fontu                           */
  BYTE width, height;          /* Szerokosc i wysokosc fontu                */
  BYTE type;                   /* Typ fontu : MONO_FNT lub COLOR_FNT        */
  BYTE *data;                  /* Dane fontu                                */
}font;

typedef struct {
  DWORD stamp;      /* MFNT                                                */
  BYTE  width;      /* Szerokosc ( musi byc podzielna przez 8 )            */
  BYTE  height;     /* Wysokosc                                            */
  BYTE  type;       /* Typ fontu : MONO_FNT lub COLOR_FNT.                 */
  BYTE  ispal;      /* Czy istnieje paleta w pliku (PAL_EXIST).            */
  BYTE  exist[32];  /* Czy dany znak istnieje -> 32 * 8 bitow = 256 znakow */
}mfn_header;

#pragma pack(1)
typedef struct {
 char VESASignature[4] PACKED;    /* 'VESA'                                 */
 short VESAVersion PACKED;        /* Numer wersji VBE                       */
 char *OEMStringPtr PACKED;       /* Wskaznik od ciagu OEM                  */
 long Capabilities PACKED;        /* Wlasciwosci karty                      */
 WORD *VideoModePtr PACKED;       /* Wskaznik do trybow video               */
 short TotalMemory PACKED;        /* Wielkosc pamieci video w 64kb blokach  */

 /* Ponizsze struktury sa od VESY 2.0                                       */
 WORD  OEMSoftwareRev PACKED;     /* Wersja softu                           */
 char  *OEMVendorNamePtr PACKED;  /* Wskaznik do ciagu producenta BIOSU     */
 char  *OEMProductNamePtr PACKED; /* Nazwa drivera VESA                     */
 char  *OEMProductRevPtr PACKED;  /* Wersja drivera VESA ( ciag )           */
 char reserved[222] PACKED;       /* Nie uzywane                            */
 char reserved2[256] PACKED;      /* Dane OEM ( od VESA 2.0 )               */
} VBE_VgaInfo;

typedef struct {
 short ModeAttributes PACKED;     /* Atrybuty trybu video                   */
 char WinAAttributes PACKED;      /* Atrybuty okna A                        */
 char WinBAttributes PACKED;      /* Atrybuty okna B                        */
 short WinGranularity PACKED;     /* Granularity w kb                       */
 short WinSize PACKED;            /* Wielkosc okna w kb                     */
 short WinASegment PACKED;        /* Segment adresu okna A                  */
 short WinBSegment PACKED;        /* Segment adresu okna B                  */
 void *WinFuncPtr PACKED;         /* Wskaznik do funcji zmieniajacej banki  */
 short BytesPerScanLine PACKED;   /* Liczba bajtow na jedna linie obrazu    */

 short XResolution PACKED;        /* Szerokosc                              */
 short YResolution PACKED;        /* Wysokosc                               */
 char XCharSize PACKED;           /* Szerokosc znaku                        */
 char YCharSize PACKED;           /* Wysokosc znaku                         */
 char NumberOfPlanes PACKED;      /* Ilosc planow obrazu                    */
 char BitsPerPixel PACKED;        /* Ilosc bitow na piksel                  */
 char NumberOfBanks PACKED;       /* Ilosc bankow                           */
 char MemoryModel PACKED;         /* Model pamieci obrazu                   */
 char BankSize PACKED;            /* Rozmiar banku                          */
 char NumberOfImagePages PACKED;  /* Ilosc stron pamieci obrazu             */
 char res1 PACKED;                /* Zarezerwowane                          */
 char RedMaskSize PACKED;         /* Liczba bitow na skladowa czerwona      */
 char RedFieldPosition PACKED;    /* Pozycja skladowej czerwonej            */
 char GreenMaskSize PACKED;       /* Liczba bitow na skladowa zielona       */
 char GreenFieldPosition PACKED;  /* Pozycja skladowej zielonej             */
 char BlueMaskSize PACKED;        /* Liczba bitow na skladowa niebieska     */
 char BlueFieldPosition PACKED;   /* Pozycja skladowej niebieskiej          */
 char RsvdMaskSize PACKED;        /* Liczba bitow na dodatkowa skladowa     */
 char RsvdFieldPosition PACKED;   /* Pozycja dodatkowej skladowej           */
 char DirectColorModeInfo PACKED; /* Atrybuty koloru                        */

 /* VESA 2.0 */
 long PhysBasePtr PACKED;         /* Adres LFB                              */
 long OffScreenMemOffset PACKED;  /* Adres poza LFB                         */
 short OffScreenMemSize PACKED;   /* Wielkosc LFB w kb                      */
 char res2[206] PACKED;           /* Zarezerwowane                          */
} VBE_ModeInfo;
#pragma pack()

/**/
/*                          Definicje stalych                               */
/**/
#define PAL_NOT_EXIST 0
#define PAL_EXIST     1
#define MONO_FNT      0
#define COLOR_FNT     1

#define BI_RGB        0
#define BI_RLE8       1
#define BI_RLE4       2

#define LFB           0x80

#define MODE_TEXT          0x03
#define MODE_320x200x256   0x13

#define MODE_640x400x256   0x100
#define MODE_640x480x256   0x101
#define MODE_800x600x256   0x103
#define MODE_1024x768x256  0x105
#define MODE_1280x1024x256 0x107

#define MODE_320x200x32K   0x10d
#define MODE_640x480x32K   0x110
#define MODE_800x600x32K   0x113
#define MODE_1024x768x32K  0x116
#define MODE_1280x1024x32K 0x119

#define MODE_320x200x64K   0x10e
#define MODE_640x480x64K   0x111
#define MODE_800x600x64K   0x114
#define MODE_1024x768x64K  0x117
#define MODE_1280x1024x64K 0x11a

#define MODE_320x200x16M   0x10f
#define MODE_640x480x16M   0x112
#define MODE_800x600x16M   0x115
#define MODE_1024x768x16M  0x118
#define MODE_1280x1024x16M 0x11b

/**/
/*                          Definicje bledow                                */
/**/
/* ====================== Definicje bledow ogolnych ======================= */
#define NO_ERROR             0   /* Brak bledu                              */
#define ERR_NO_MEM         255   /* Blad braku pamieci                      */

/* ====================== Definicje bledow engine'a ====================== */
#define ERR_NO_MZL         254   /* Plik nie jest w formacie MZL            */
#define ERR_NO_VERT        253   /* Brak danych o wierzcholkach             */
#define ERR_NO_FACES       252   /* Brak danych o scianach                  */

#define ERR_TOO_MANY_OBJ   251   /* Za duzo obiektow w scenie               */
#define ERR_TOO_MANY_FACES 250   /* Za duzo scian w scenie                  */
#define ERR_NO_OBJ         249   /* Brak obiektu o podanym numerze          */

/* Te definicje powinny zostac usuniete                                     */
/* Program powinien obliczyc normalne                                       */
#define ERR_NO_VERT_NORM   203   /* Brak danych o normalnych wierzcholkow   */
#define ERR_NO_FACE_NORM   202   /* Brak danych o normalnych scian          */
#define ERR_NO_TEXT        201   /* Brak danych o koordynatach tekstur      */

/* ==================== Definicje bledow graficznych ====================== */
#define ERR_NO_BMP         148   /* Plik nie jest w formacie BMP            */
#define ERR_BMP_COMPRESSED 147   /* BMP jest skompresowany                  */
#define ERR_NO_256_COLORS  146   /* Ilosc kolorow jest rozna od 256         */
#define ERR_NO_PCX         145   /* Plik nie jest w formacie PCX            */
#define ERR_BAD_PCX_VER    144   /* Zla wersja PCXa ( rozna od 5 )          */
#define ERR_NO_LBM         143   /* Plik nie jest w formacie LBM            */
#define ERR_BAD_LBM        142   /* Zla wersja pliku LBM ( nie PBM )        */
#define ERR_NO_MFN         141   /* Plik nie jest w formacie MFN            */

#define ERR_NO_VESA        140
#define ERR_NO_FUNC        139
#define ERR_NO_VESA_MODE   138
#define ERR_NO_LFB         137
#define ERR_GENERIC_VESA   136

/**/
/*                          Definicje klawiszy                              */
/**/
#define K_ESCAPE      1
#define K_1           2
#define K_2           3
#define K_3           4
#define K_4           5
#define K_5           6
#define K_6           7
#define K_7           8
#define K_8           9
#define K_9          10
#define K_0          11
#define K_MINUS      12
#define K_EQUALS     13
#define K_BACKSPACE  14
#define K_TAB        15
#define K_Q          16
#define K_W          17
#define K_E          18
#define K_R          19
#define K_T          20
#define K_Y          21
#define K_U          22
#define K_I          23
#define K_O          24
#define K_P          25
#define K_LBRACKET   26
#define K_RBRACKET   27
#define K_ENTER      28
#define K_CTRL       29
#define K_A          30
#define K_S          31
#define K_D          32
#define K_F          33
#define K_G          34
#define K_H          35
#define K_J          36
#define K_K          37
#define K_L          38
#define K_SEMICOLON  39
#define K_QUOTE      40
#define K_TILDE      41
#define K_LSHIFT     42
#define K_BACKSLASH  43
#define K_Z          44
#define K_X          45
#define K_C          46
#define K_V          47
#define K_B          48
#define K_N          49
#define K_M          50
#define K_COMMA      51
#define K_PERIOD     52
#define K_SLASH      53
#define K_RSHIFT     54
#define K_STAR_NUM   55
#define K_ALT        56
#define K_SPACE      57
#define K_CAPSLOCK   58
#define K_F1         59
#define K_F2         60
#define K_F3         61
#define K_F4         62
#define K_F5         63
#define K_F6         64
#define K_F7         65
#define K_F8         66
#define K_F9         67
#define K_F10        68
#define K_NUMLOCK    69
#define K_SCROLL     70
#define K_HOME       71
#define K_UP         72
#define K_PAGEUP     73
#define K_MINUS_NUM  74
#define K_LEFT       75
#define K_CENTER     76
#define K_RIGHT      77
#define K_PLUS_NUM   78
#define K_END        79
#define K_DOWN       80
#define K_PAGEDN     81
#define K_INSERT     82
#define K_DELETE     83
#define K_F11        87
#define K_F12        88

/**/
/*                   Zmienne zdefiniowane w bibliotece                      */
/**/
extern BYTE  *virtual_screen;    /* Wirtualny ekran                         */
extern BYTE  *screen;            /* Fizyczny ekran                          */
extern int   circ_sin[256];      /* Tablica sinusa do rysowania okregu      */
extern int   circ_cos[256];      /* Tablica cosinusa do rysowania okregu    */
extern int   CLIP_MIN_X;         /* Minimalna wartosc X do clippingu        */
extern int   CLIP_MAX_X;         /* Maksymalna wartosc X do clippingu       */
extern int   CLIP_MIN_Y;         /* Minimalna wartosc Y do clippingu        */
extern int   CLIP_MAX_Y;         /* Maksymalna wartosc Y do clippingu       */
extern BYTE  volatile raw_key;   /* Ostatnio nacisniety klawisz             */
extern BYTE  volatile keyboard[128]; /* Bufor handlera klawiatury           */
extern int   video_mode;         /* Aktualnie ustawiony tryb video          */
extern int   WIDTH, HEIGHT;      /* Aktualna szerokosc i wysokosc ekranu    */
extern VBE_VgaInfo  vesa_info;   /* VESA info                               */
extern VBE_ModeInfo mode_info;   /* VESA mode info                          */
extern WORD  current_bank;       /* Aktualnie ustawiony bank VESA           */
extern int   bytes_per_line;     /* Liczba bajtow na 1 linie                */
extern int   win_granularity;    /* VESA bank granularity                   */
extern DWORD LFB_address;        /* Adress LFB lub NULL, gdy nie ma LFB     */
extern BYTE  video_pages;        /* Ilosc stron video ( tylko jesli LFB )   */
extern int   YLookUp[1024];      /* Y * row LookUp Table                    */
extern int   X_CENTRE, Y_CENTRE; /* Srodek ekranu                           */
extern int   PAGE_SIZE;          /* Wielkosc ekranu w bajtach               */
extern BYTE  page_on_screen;     /* Aktualnie wyswietlana na ekranie strona */

/**/
/*                           Funkcje biblioteki                             */
/**/
/* ======================= Funkcje z modulu draw.c ======================== */
void clr_virt_scr(BYTE color);
void put_pixel(int x, int y, BYTE color);
BYTE get_pixel(int x, int y);
void circle(int xs, int ys, int r, BYTE color);
void line(int x1, int y1, int x2, int y2, BYTE col);
void out_text(BYTE *where, int x, int y, char *text, font f, BYTE col);
BYTE get_image(bitmap *dst, int x, int y, int width, int height);
void put_image(bitmap src, int x, int y);
void put_image_transparent(bitmap src, int x, int y);

/* ===================== Funkcje z modulu tri_fill.c ====================== */
void flat_hline(int y, int x1, int x2, BYTE color);
void flat_triangle(int x1, int y1, int x2, int y2, int x3, int y3, BYTE color);
void gouraud_hline(int y, int x1, int x2, BYTE c1, BYTE c2);
void gouraud_triangle(int x1, int y1, int x2, int y2, int x3, int y3,
                      BYTE c1, BYTE c2, BYTE c3);
void texture_hline(int y, int x1, int x2,
                   BYTE u1, BYTE v1, BYTE u2, BYTE v2, BYTE *tex);
void texture_triangle(int x1, int y1, int x2, int y2, int x3, int y3,
                      BYTE u1, BYTE v1, BYTE u2, BYTE v2, BYTE u3, BYTE v3,
                      BYTE *tex);

/* ===================== Funkcje z modulu palette.c ======================= */
void set_dac_color(BYTE nr, BYTE r, BYTE g, BYTE b );
void set_dac_colorRGB(BYTE nr, RGB col);
void smooth_RGB(BYTE start_num, BYTE end_num, RGB start_col, RGB end_col, RGB *pal);
void set_palette(RGB *pal);
void set_black_palette(void);

/* ===================== Funkcje z modulu loaders.c ======================= */
BYTE Load_BMP(char *fname, bitmap *buf, RGB *pal, DWORD offset);
BYTE Load_PCX(char *fname, bitmap *buf, RGB *pal, DWORD offset, DWORD fsize);
BYTE Load_LBM(char *fname, bitmap *buf, RGB *pal);
BYTE Load_RAW(char *fname, bitmap *buf, int width, int height);
BYTE Load_font(char *fname, font *buf, RGB *pal);

/* ===================== Funkcje z modulu khandler.c ====================== */
void Init_handler(void);
void Remove_handler(void);

/* ===================== Funkcje z modulu my_vga13.c ====================== */
void (*swap_pages)(void);
void swap_pages_13h(void);
void wait_VBL(void);
void set_clip(int min_x, int min_y, int max_x, int max_y);
void set_video_mode(BYTE mode);
BYTE init_gfx(int mode, BYTE info);
void deinit_gfx(void);
void Print_error(char *text, BYTE err);

/* ===================== Funkcje z modulu vesa.c ========================== */
int  VBE_detect(VBE_VgaInfo *vbeinfo);
BYTE VBE_get_mode_info(WORD mode, VBE_ModeInfo *modeinfo);
BYTE VBE_set_mode(WORD mode);
BYTE VBE_set_bank(WORD bank);
BYTE VBE_set_display_start(int scanline, int pixel);
BYTE VBE_get_linear_address(VBE_ModeInfo *minfo);
void VBE_swap_pages(void);

#endif
